/*---------------------------------------------------------------------------
    demo_exchange.h - demo_exchange component

    This class implements the ASL demo server exchange class (this is a
    fanout exchange).
    Generated from demo_exchange.icl by smt_object_gen using GSL/4.
    
    Copyright (c) 1996-2009 iMatix Corporation
    All rights reserved.
    
    This file is licensed under the BSD license as follows:
    
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the
      distribution.
    * Neither the name of iMatix Corporation nor the names of its
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.
    
    THIS SOFTWARE IS PROVIDED BY IMATIX CORPORATION "AS IS" AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
    PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL IMATIX CORPORATION BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
    BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    These header files use macros to implement a split inclusion in which all
    safe definitions (those that do not depend on the presence of other
    definitions) are done first, and all unsafe definitions are done in a
    second pass through the same headers.

    The first header file included from the main C program defines itself as
    the "root" and thus controls the inclusion of the safe/unsafe pieces of
    the other headers.
 *---------------------------------------------------------------------------*/

#if !defined (ICL_IMPORT_HEADERS) || (ICL_IMPORT_HEADERS == 1)
# ifndef INCLUDE_DEMO_EXCHANGE_SAFE
#   define INCLUDE_DEMO_EXCHANGE_SAFE
#   define INCLUDE_DEMO_EXCHANGE_ACTIVE
#   if !defined (ICL_IMPORT_HEADERS)
#       define ICL_IMPORT_DEMO_EXCHANGE_ROOT
#       define ICL_IMPORT_HEADERS 1
#   endif
#   ifdef __cplusplus
extern "C" {
#   endif

//  Our own class typedef

typedef struct _demo_exchange_t demo_exchange_t;
#if defined (DEBUG) || defined (BASE_HISTORY) || defined (BASE_HISTORY_DEMO_EXCHANGE)
#    define DEMO_EXCHANGE_HISTORY_LENGTH 32
#endif

#define DEMO_EXCHANGE_ALIVE             0xFABB
#define DEMO_EXCHANGE_DEAD              0xDEAD

#define DEMO_EXCHANGE_ASSERT_SANE(self)\
{\
    if (!self) {\
        fprintf (stderr, "FATAL ERROR at %s:%u, in %s\n", __FILE__, __LINE__, ICL_ASSERT_SANE_FUNCTION);\
        fprintf (stderr, "Attempting to work with a NULL demo_exchange\n");\
        fprintf (stderr, "Please report this to openamq-dev@lists.openamq.org\n");\
        fflush (stderr);\
        abort ();\
    }\
    if (self->object_tag != DEMO_EXCHANGE_ALIVE) {\
        fprintf (stderr, "FATAL ERROR at %s:%u, in %s\n", __FILE__, __LINE__, ICL_ASSERT_SANE_FUNCTION);\
        fprintf (stderr, "demo_exchange at %p expected object_tag=0x%x, actual object_tag=0x%x\n", self, DEMO_EXCHANGE_ALIVE, self->object_tag);\
        demo_exchange_show (self, ICL_CALLBACK_DUMP, stderr);\
        fprintf (stderr, "Please report this to openamq-dev@lists.openamq.org\n");\
        fflush (stderr);\
        abort ();\
    }\
}
#   ifdef __cplusplus
}
#   endif

#   include "demo_queue_list.h"
#   include "demo_server_classes.h"
#   include "demo_exchange_table.h"
#   include "icl_mem.h"
#   ifdef __cplusplus
extern "C" {
#   endif
#include "icl.h"
#include "ipr.h"
#include "smt.h"
#include "demo_exchange_agent.h"

#   ifdef __cplusplus
}
#   endif
#   undef INCLUDE_DEMO_EXCHANGE_ACTIVE
#   if defined (ICL_IMPORT_DEMO_EXCHANGE_ROOT)
#       undef  ICL_IMPORT_HEADERS
#       define ICL_IMPORT_HEADERS 2
#   endif
# endif
#endif

#if !defined (ICL_IMPORT_HEADERS) || (ICL_IMPORT_HEADERS == 2)
# ifndef INCLUDE_DEMO_EXCHANGE_UNSAFE
#   define INCLUDE_DEMO_EXCHANGE_UNSAFE
#   define INCLUDE_DEMO_EXCHANGE_ACTIVE

#   include "demo_queue_list.h"
#   include "demo_server_classes.h"
#   include "demo_exchange_table.h"
#   include "icl_mem.h"

#   ifdef __cplusplus
extern "C" {
#   endif
#include "icl.h"
#include "ipr.h"
#include "smt.h"
#include "demo_exchange_agent.h"
//  Global variables

extern Bool
    demo_exchange_animating;

//  Structure of the demo_exchange object

struct _demo_exchange_t {
    volatile int  
        links;                          //  Number of links to the item
    volatile qbyte
        zombie;                         //  Object destroyed but not freed
#if defined (DEBUG) || defined (BASE_HISTORY) || defined (BASE_HISTORY_DEMO_EXCHANGE)
    //  Possession history, for tracing
    volatile qbyte
        history_last;
    char
        *history_file [DEMO_EXCHANGE_HISTORY_LENGTH];
    int
        history_line  [DEMO_EXCHANGE_HISTORY_LENGTH];
    char
        *history_type [DEMO_EXCHANGE_HISTORY_LENGTH];
    int
        history_links [DEMO_EXCHANGE_HISTORY_LENGTH];
#endif
    dbyte
        object_tag;                     //  Object validity marker
smt_thread_t
    *thread;
demo_exchange_table_t
    *table_head;                    //  Hash table container
demo_exchange_t
    *table_next;                    //  Next item in the hash slot
qbyte
    table_index;                    //  Index of item in table
icl_shortstr_t
    key;                            //  Item's original key
char
    *name;                          //  Exchange name
Bool
    durable;                        //  Is exchange durable?
Bool
    auto_delete;                    //  Auto-delete unused exchange?
demo_queue_list_t
    *queue_list;                    //  List of bound queues
};

#   ifdef __cplusplus
}
#   endif
#   ifdef __cplusplus
extern "C" {
#   endif
//  Method prototypes
#define demo_exchange_new(name,durable,auto_delete)  demo_exchange_new_ (__FILE__, __LINE__, name, durable, auto_delete)
demo_exchange_t *
    demo_exchange_new_ (
char * file,                            //  Source file for call
size_t line,                            //  Line number for call
char * name,                            //  Exchange name
Bool durable,                           //  Is exchange durable?
Bool auto_delete                        //  Auto-delete unused exchange?
);

demo_exchange_t *
    demo_exchange_search (
char * name                             //  Exchange name
);

int
    demo_exchange_bind (
demo_exchange_t * self,                 //  Reference to object
demo_server_channel_t * channel,        //  The channel in question
demo_queue_t * queue,                   //  The queue to bind
icl_longstr_t * arguments               //  Bind arguments
);

int
    demo_exchange_publish (
demo_exchange_t * self,                 //  Reference to object
demo_server_channel_t * channel,        //  The channel in question
demo_content_basic_t * content          //  The message content
);

void
    demo_exchange_terminate (
void);

void
    demo_exchange_selftest (
void);

void
    demo_exchange_remove_from_all_containers (
demo_exchange_t * self                  //  The item
);

#define demo_exchange_show(item,opcode,trace_file)  demo_exchange_show_ (item, opcode, trace_file, __FILE__, __LINE__)
void
    demo_exchange_show_ (
void * item,                            //  The opaque pointer
int opcode,                             //  The callback opcode
FILE * trace_file,                      //  File to print to
char * file,                            //  Source file
size_t line                             //  Line number
);

#define demo_exchange_destroy(self)     demo_exchange_destroy_ (self, __FILE__, __LINE__)
int
    demo_exchange_destroy_ (
demo_exchange_t * ( * self_p ),         //  Reference to object reference
char * file,                            //  Source file
size_t line                             //  Line number
);

#define demo_exchange_unlink(self)      demo_exchange_unlink_ (self, __FILE__, __LINE__)
void
    demo_exchange_unlink_ (
demo_exchange_t * ( * self_p ),         //  Reference to object reference
char * file,                            //  Source file for call
size_t line                             //  Line number for call
);

void
    demo_exchange_show_animation (
Bool enabled                            //  Are we enabling or disabling animation?
);

#define demo_exchange_free(self)        demo_exchange_free_ (self, __FILE__, __LINE__)
void
    demo_exchange_free_ (
demo_exchange_t * self,                 //  Object reference
char * file,                            //  Source file
size_t line                             //  Line number
);

#define demo_exchange_link(self)        demo_exchange_link_ (self, __FILE__, __LINE__)
demo_exchange_t *
    demo_exchange_link_ (
demo_exchange_t * self,                 //  Not documented
char * file,                            //  Source file for call
size_t line                             //  Line number for call
);

#define demo_exchange_new_in_scope(self_p,_scope,name,durable,auto_delete)  demo_exchange_new_in_scope_ (self_p, _scope, __FILE__, __LINE__, name, durable, auto_delete)
void
    demo_exchange_new_in_scope_ (
demo_exchange_t * * self_p,             //  Not documented
icl_scope_t * _scope,                   //  Not documented
char * file,                            //  Source file for call
size_t line,                            //  Line number for call
char * name,                            //  Exchange name
Bool durable,                           //  Is exchange durable?
Bool auto_delete                        //  Auto-delete unused exchange?
);


#   ifdef __cplusplus
}
#   endif
#   undef INCLUDE_DEMO_EXCHANGE_ACTIVE
#   if defined (ICL_IMPORT_DEMO_EXCHANGE_ROOT)
#       undef  ICL_IMPORT_HEADERS
#       define ICL_IMPORT_HEADERS 3
#   endif
# endif
#endif

#if !defined (ICL_IMPORT_HEADERS) || (ICL_IMPORT_HEADERS == 3)
# if !defined (INCLUDE_DEMO_EXCHANGE_INLINE)
#   define INCLUDE_DEMO_EXCHANGE_INLINE
#   define INCLUDE_DEMO_EXCHANGE_ACTIVE

#   include "demo_queue_list.h"
#   include "demo_server_classes.h"
#   include "demo_exchange_table.h"
#   include "icl_mem.h"

#   ifdef __cplusplus
extern "C" {
#   endif

#if (defined (BASE_TRACE)   || defined (BASE_TRACE_DEMO_EXCHANGE)   || defined (BASE_TRACE_DEMO_EXCHANGE_NEW)   || defined (BASE_TRACE_DEMO_EXCHANGE_DESTROY)   || defined (BASE_TRACE_DEMO_EXCHANGE_SEARCH)   || defined (BASE_TRACE_DEMO_EXCHANGE_BIND)   || defined (BASE_TRACE_DEMO_EXCHANGE_PUBLISH)   || defined (BASE_TRACE_DEMO_EXCHANGE_INITIALISE)   || defined (BASE_TRACE_DEMO_EXCHANGE_TERMINATE)   || defined (BASE_TRACE_DEMO_EXCHANGE_SELFTEST)   || defined (BASE_TRACE_DEMO_EXCHANGE_REMOVE_FROM_ALL_CONTAINERS)   || defined (BASE_TRACE_DEMO_EXCHANGE_SHOW)   || defined (BASE_TRACE_DEMO_EXCHANGE_DESTROY_PUBLIC)   || defined (BASE_TRACE_DEMO_EXCHANGE_UNLINK)   || defined (BASE_TRACE_DEMO_EXCHANGE_SHOW_ANIMATION)   || defined (BASE_TRACE_DEMO_EXCHANGE_FREE)   || defined (BASE_TRACE_DEMO_EXCHANGE_ALLOC)   || defined (BASE_TRACE_DEMO_EXCHANGE_LINK)   || defined (BASE_TRACE_DEMO_EXCHANGE_CACHE_INITIALISE)   || defined (BASE_TRACE_DEMO_EXCHANGE_CACHE_PURGE)   || defined (BASE_TRACE_DEMO_EXCHANGE_CACHE_TERMINATE)   || defined (BASE_TRACE_DEMO_EXCHANGE_NEW_IN_SCOPE) )
void
    demo_exchange_dump (icl_os_thread_t thread, apr_time_t time, qbyte info);
#endif

#include "icl.h"
#include "ipr.h"
#include "smt.h"
#include "demo_exchange_agent.h"
#   ifdef __cplusplus
}
#   endif
#   undef INCLUDE_DEMO_EXCHANGE_ACTIVE
#   if defined (ICL_IMPORT_DEMO_EXCHANGE_ROOT)
#       undef ICL_IMPORT_HEADERS
#       undef ICL_IMPORT_DEMO_EXCHANGE_ROOT
#   endif
# endif
#endif
