/*---------------------------------------------------------------------------
    ipr_crc.h - ipr_crc component

    The CRC module calculates a CCITT cyclic-redundancy code for a
    block of data or a series of data.  Use to calculate the check
    sum for a block of data or a series of data items.  Works with
    data and strings.
    Generated from ipr_crc.icl by icl_gen using GSL/4.
    
    Copyright (c) 1996-2009 iMatix Corporation
    All rights reserved.
    
    This file is licensed under the BSD license as follows:
    
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the
      distribution.
    * Neither the name of iMatix Corporation nor the names of its
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.
    
    THIS SOFTWARE IS PROVIDED BY IMATIX CORPORATION "AS IS" AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
    PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL IMATIX CORPORATION BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
    BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    These header files use macros to implement a split inclusion in which all
    safe definitions (those that do not depend on the presence of other
    definitions) are done first, and all unsafe definitions are done in a
    second pass through the same headers.

    The first header file included from the main C program defines itself as
    the "root" and thus controls the inclusion of the safe/unsafe pieces of
    the other headers.
 *---------------------------------------------------------------------------*/

#if !defined (ICL_IMPORT_HEADERS) || (ICL_IMPORT_HEADERS == 1)
# ifndef INCLUDE_IPR_CRC_SAFE
#   define INCLUDE_IPR_CRC_SAFE
#   define INCLUDE_IPR_CRC_ACTIVE
#   if !defined (ICL_IMPORT_HEADERS)
#       define ICL_IMPORT_IPR_CRC_ROOT
#       define ICL_IMPORT_HEADERS 1
#   endif
#   ifdef __cplusplus
extern "C" {
#   endif

//  Our own class typedef

typedef struct _ipr_crc_t ipr_crc_t;

#define IPR_CRC_ALIVE                   0xFABB
#define IPR_CRC_DEAD                    0xDEAD

#define IPR_CRC_ASSERT_SANE(self)\
{\
    if (!self) {\
        fprintf (stderr, "FATAL ERROR at %s:%u, in %s\n", __FILE__, __LINE__, ICL_ASSERT_SANE_FUNCTION);\
        fprintf (stderr, "Attempting to work with a NULL ipr_crc\n");\
        fprintf (stderr, "Please report this to openamq-dev@lists.openamq.org\n");\
        fflush (stderr);\
        abort ();\
    }\
    if (self->object_tag != IPR_CRC_ALIVE) {\
        fprintf (stderr, "FATAL ERROR at %s:%u, in %s\n", __FILE__, __LINE__, ICL_ASSERT_SANE_FUNCTION);\
        fprintf (stderr, "ipr_crc at %p expected object_tag=0x%x, actual object_tag=0x%x\n", self, IPR_CRC_ALIVE, self->object_tag);\
        ipr_crc_show (self, ICL_CALLBACK_DUMP, stderr);\
        fprintf (stderr, "Please report this to openamq-dev@lists.openamq.org\n");\
        fflush (stderr);\
        abort ();\
    }\
}
#   ifdef __cplusplus
}
#   endif

#   include "icl_mem.h"
#   ifdef __cplusplus
extern "C" {
#   endif

#   ifdef __cplusplus
}
#   endif
#   undef INCLUDE_IPR_CRC_ACTIVE
#   if defined (ICL_IMPORT_IPR_CRC_ROOT)
#       undef  ICL_IMPORT_HEADERS
#       define ICL_IMPORT_HEADERS 2
#   endif
# endif
#endif

#if !defined (ICL_IMPORT_HEADERS) || (ICL_IMPORT_HEADERS == 2)
# ifndef INCLUDE_IPR_CRC_UNSAFE
#   define INCLUDE_IPR_CRC_UNSAFE
#   define INCLUDE_IPR_CRC_ACTIVE

#   include "icl_mem.h"

#   ifdef __cplusplus
extern "C" {
#   endif
//  Global variables

extern Bool
    ipr_crc_animating;

//  Structure of the ipr_crc object

struct _ipr_crc_t {
dbyte
    object_tag;                     //  Object validity marker
qbyte
    value;
};

#   ifdef __cplusplus
}
#   endif
#   ifdef __cplusplus
extern "C" {
#   endif
//  Method prototypes
int
    ipr_crc_calc (
ipr_crc_t * self,                       //  Reference to object
byte * data,                            //  Data to checksum
size_t size                             //  Length of data
);

int
    ipr_crc_calc_str (
ipr_crc_t * self,                       //  Reference to object
char * string                           //  String to checksum
);

void
    ipr_crc_selftest (
void);

void
    ipr_crc_terminate (
void);

#define ipr_crc_new()                   ipr_crc_new_ (__FILE__, __LINE__)
ipr_crc_t *
    ipr_crc_new_ (
char * file,                            //  Source file for call
size_t line                             //  Line number for call
);

#define ipr_crc_show(item,opcode,trace_file)  ipr_crc_show_ (item, opcode, trace_file, __FILE__, __LINE__)
void
    ipr_crc_show_ (
void * item,                            //  The opaque pointer
int opcode,                             //  The callback opcode
FILE * trace_file,                      //  File to print to
char * file,                            //  Source file
size_t line                             //  Line number
);

#define ipr_crc_destroy(self)           ipr_crc_destroy_ (self, __FILE__, __LINE__)
void
    ipr_crc_destroy_ (
ipr_crc_t * ( * self_p ),               //  Reference to object reference
char * file,                            //  Source fileSource file
size_t line                             //  Line numberLine number
);

void
    ipr_crc_show_animation (
Bool enabled                            //  Are we enabling or disabling animation?
);

#define ipr_crc_new_in_scope(self_p,_scope)  ipr_crc_new_in_scope_ (self_p, _scope, __FILE__, __LINE__)
void
    ipr_crc_new_in_scope_ (
ipr_crc_t * * self_p,                   //  Not documented
icl_scope_t * _scope,                   //  Not documented
char * file,                            //  Source file for call
size_t line                             //  Line number for call
);


#   ifdef __cplusplus
}
#   endif
#   undef INCLUDE_IPR_CRC_ACTIVE
#   if defined (ICL_IMPORT_IPR_CRC_ROOT)
#       undef  ICL_IMPORT_HEADERS
#       define ICL_IMPORT_HEADERS 3
#   endif
# endif
#endif

#if !defined (ICL_IMPORT_HEADERS) || (ICL_IMPORT_HEADERS == 3)
# if !defined (INCLUDE_IPR_CRC_INLINE)
#   define INCLUDE_IPR_CRC_INLINE
#   define INCLUDE_IPR_CRC_ACTIVE

#   include "icl_mem.h"

#   ifdef __cplusplus
extern "C" {
#   endif

#if (defined (BASE_TRACE)   || defined (BASE_TRACE_IPR_CRC)   || defined (BASE_TRACE_IPR_CRC_CALC)   || defined (BASE_TRACE_IPR_CRC_CALC_STR)   || defined (BASE_TRACE_IPR_CRC_SELFTEST)   || defined (BASE_TRACE_IPR_CRC_TERMINATE)   || defined (BASE_TRACE_IPR_CRC_NEW)   || defined (BASE_TRACE_IPR_CRC_SHOW)   || defined (BASE_TRACE_IPR_CRC_DESTROY_PUBLIC)   || defined (BASE_TRACE_IPR_CRC_DESTROY)   || defined (BASE_TRACE_IPR_CRC_ALLOC)   || defined (BASE_TRACE_IPR_CRC_FREE)   || defined (BASE_TRACE_IPR_CRC_CACHE_INITIALISE)   || defined (BASE_TRACE_IPR_CRC_CACHE_PURGE)   || defined (BASE_TRACE_IPR_CRC_CACHE_TERMINATE)   || defined (BASE_TRACE_IPR_CRC_SHOW_ANIMATION)   || defined (BASE_TRACE_IPR_CRC_NEW_IN_SCOPE) )
void
    ipr_crc_dump (icl_os_thread_t thread, apr_time_t time, qbyte info);
#endif

#   ifdef __cplusplus
}
#   endif
#   undef INCLUDE_IPR_CRC_ACTIVE
#   if defined (ICL_IMPORT_IPR_CRC_ROOT)
#       undef ICL_IMPORT_HEADERS
#       undef ICL_IMPORT_IPR_CRC_ROOT
#   endif
# endif
#endif
